@@ -6,7 +6,7 @@ import pypandoc
 
 from tqdm import tqdm
 
-from examtool.api.utils import rand_id
+from examtool.api.utils import list_to_dict, rand_id
 
 VERSION = 2  # increment when backward-incompatible changes are made
 
@@ -134,7 +134,12 @@ def parse_define(
             blocks[i] = tuple(block[1:-1].split(" "))
         if not all(len(block) == len(blocks[0]) for block in blocks):
             raise SyntaxError("DEFINE GROUP blocks must all be of the same length")
-        substitution_groups.append(blocks)
+        substitution_groups.append(
+            {
+                "directives": blocks[0],
+                "replacements": list_to_dict([list_to_dict(block) for block in blocks]),
+            }
+        )
     else:
         substitutions[directive] = rest.split(" ")
 
@@ -146,6 +151,7 @@ def parse_input_lines(lines):
     correct_options = []
     if directive == "OPTION" or directive == "SELECT":
         options = []
+        existing_options = set()
         for line in lines:
             _, other_directive, rest = parse_directive(line)
             if other_directive != directive:
@@ -165,6 +171,10 @@ def parse_input_lines(lines):
             if is_correct:
                 correct_options.append(rest)
 
+            if rest in existing_options:
+                raise SyntaxError("Cannot have duplicate INPUT options")
+            existing_options.add(rest)
+
             options.append(parse(rest))
             options[-1]["fixed"] = is_fixed
         return (
